@@ -1,7 +1,7 @@
 /* common header file
 
   Authors: Bernd Paysan, Anton Ertl, David Kühling, Jens Wilke, Neal Crook
-  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Free Software Foundation, Inc.
+  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -300,12 +300,25 @@ typedef union {
 
 typedef Label *Xt;
 
+#define NEW_CFA /* undefine if you want old cfa */
+
+#ifdef NEW_CFA
+#define CFA_OFFSET	2
+/* PFA gives the parameter field address corresponding to a cfa */
+#define PFA(cfa)	(((Cell *)(cfa)))
+/* PFA1 is a special version for use just after a NEXT1 */
+#define PFA1(cfa)	PFA(cfa)
+/* CODE_ADDRESS is the address of the code jumped to through the code field */
+#define CODE_ADDRESS(cfa)	(((Xt)(cfa))[-CFA_OFFSET])
+#else
+#define CFA_OFFSET	0
 /* PFA gives the parameter field address corresponding to a cfa */
 #define PFA(cfa)	(((Cell *)cfa)+1)
 /* PFA1 is a special version for use just after a NEXT1 */
 #define PFA1(cfa)	PFA(cfa)
 /* CODE_ADDRESS is the address of the code jumped to through the code field */
 #define CODE_ADDRESS(cfa)	(*(Xt)(cfa))
+#endif
 
 /* DOES_CODE is the Forth code does jumps to */
 #if !defined(DOUBLY_INDIRECT)
@@ -404,7 +417,7 @@ typedef struct {
 
 typedef struct {
   Address base;
-  Address end;
+  Cell size;
   Address dp;
 } SectionHeader;
 
@@ -418,11 +431,17 @@ struct F83Name {
 #define F83NAME_COUNT(np)	((np)->countetc & 0x1f)
 #endif
 
+#ifdef NEW_CFA
+#define LONGNAME_OFF 4
+#define CFA_OFF -3
+#else
 #define LONGNAME_OFF 3
+#define CFA_OFF 0
+#endif
 #define RESERVED_BITS 8
-#define LONGNAME_COUNT(np)     ((((Cell*)np)[-LONGNAME_OFF]) & (((~((UCell)0))<<RESERVED_BITS)>>RESERVED_BITS))
+#define LONGNAME_COUNT(np)     ((((Cell*)np)[-LONGNAME_OFF]) & (~((UCell)0)>>RESERVED_BITS))
 #define LONGNAME_NAME(np)      ((Char *)(np)-LONGNAME_OFF*sizeof(Cell)-LONGNAME_COUNT(np))
-#define LONGNAME_NEXT(np)      ((struct Longname*)(((Cell*)np)[-2]))
+#define LONGNAME_NEXT(np)      ((struct Longname*)(((Cell*)np)[-LONGNAME_OFF+1]))
 
 struct Cellpair {
   Cell n1;
@@ -538,7 +557,10 @@ UCell rshift(UCell u1, UCell n);
 int gforth_system(Char *c_addr, UCell u);
 void gforth_ms(UCell u);
 UCell gforth_dlopen(Char *c_addr, UCell u);
+UCell gforth_dlopen2(Char *c_addr, UCell u);
+UCell gforth_dlsym2(Char *c_addr1, UCell u1, UCell u2);
 void gforth_dlclose(UCell lib);
+void gforth_dlclose2(UCell lib);
 Cell capscompare(Char *c_addr1, UCell u1, Char *c_addr2, UCell u2);
 int gf_ungetc(int c, FILE *stream);
 void gf_regetc(FILE *stream);
