@@ -626,11 +626,11 @@ struct Cellpair represent(Float r, Address c_addr1, UCell u, Cell *np)
     ok = 0;
     repstr("NaN",c_addr,u);
   } else {
-    sign = FLAG(r<0);
+    sign = FLAG(signbit(r)!=0);
     if (isinf(r)) {
       decpt=0;
       ok = 0;
-      if (r<0 && u>0) {
+      if (sign && u>0) {
 	*c_addr++ = '-';
 	u--;
       }
